#include <cstdio>
#include <vector>

#define MAX_N 500000

std::vector<int> graph[MAX_N + 1];
bool visited[MAX_N + 1];
int path_len[MAX_N + 1];
int path_cnt[MAX_N + 1];

int cycle[MAX_N];
int cycle_len = 0;

int ans_len = 0;
long long ans_cnt = 0;

void del_dir_edge(int a, int b) {
	for (int &a_n : graph[a]) {
		if (a_n == b) {
			a_n = graph[a].back();
			graph[a].pop_back();
			return;
		}
	}
}

void del_edge(int a, int b) {
	del_dir_edge(a, b);
	del_dir_edge(b, a);
}

int fetch_cycle_dfs(int p, int v) {
	int res = 0;
	visited[v] = true;
	for (int v_n : graph[v]) {
		if (v_n == p) continue;
		if (visited[v_n])
			res = v_n;
		else
			res = fetch_cycle_dfs(v, v_n);

		if (res != 0) {
			del_edge(v, v_n);
			cycle[cycle_len++] = v;
			if (res == v)
				res = 0;
			return res;
		}
	}
	return 0;
}

void fetch_cycle() {
	fetch_cycle_dfs(0, 1);
}

void del_parent_edges_dfs(int p, int v) {
	del_dir_edge(v, p);
	for (int v_n : graph[v]) {
		del_parent_edges_dfs(v, v_n);
	}
}

void del_parent_edges(int v) {
	del_parent_edges_dfs(0, v);
}

void get_paths_dfs(int v) {
	path_len[v] = 0;
	path_cnt[v] = 1;
	for (int v_n : graph[v]) {
		get_paths_dfs(v_n);
		if (path_len[v] < path_len[v_n] + 1) {
			path_len[v] = path_len[v_n] + 1;
			path_cnt[v] = path_cnt[v_n];
		}
		else if (path_len[v] == path_len[v_n] + 1) {
			path_cnt[v] += path_cnt[v_n];
		}
	}
}

void get_paths(int v) {
	get_paths_dfs(v);
}

void register_paths(int len, long long cnt) {
	if (cnt == 0) return;
	if (ans_len < len) {
		ans_len = len;
		ans_cnt = cnt;
	}
	else if (ans_len == len) {
		ans_cnt += cnt;
	}
}

void register_subtree(int v) {
	int len1 = -1;
	int len2 = -1;
	long long cnt1 = 0;

	for (int v_n : graph[v]) {
		int l = path_len[v_n];
		int c = path_cnt[v_n];
		if (l >= len1) {
			len2 = len1;
			len1 = l;
			cnt1 = c;
		}
		else if (l >= len2) {
			len2 = l;
		}
	}

	long long sum = 0;
	long long cnt = 0;
	for (int v_n : graph[v]) {
		int l = path_len[v_n];
		int c = path_cnt[v_n];
		if (l == len2) {
			if (len1 == len2)
				cnt += sum * c;
			else
				cnt += cnt1 * c;
			sum += c;
		}
	}

	register_paths(path_len[v], path_cnt[v]);
	register_paths(len1 + len2 + 2, cnt);
}

struct queue_elem {
	int skipped;
	int len;
	long long cnt;
};

queue_elem queue[MAX_N + MAX_N / 2];
int queue_head = 0;
int queue_tail = 0;
int queue_add = 1;
int queue_sub = 0;
int queue_curr_len = 0;
long long queue_curr_sum = 0;

void queue_push(int len, long long cnt) {
	int skipped = 0;

	len += queue_add;
	queue_add++;

	while (queue_tail > queue_head && queue[queue_tail-1].len < len) {
		skipped += queue[queue_tail-1].skipped + 1;
		queue_tail--;
	}

	queue[queue_tail].skipped = skipped;
	queue[queue_tail].len = len;
	queue[queue_tail].cnt = cnt;
	queue_tail++;

	if (queue_curr_len == len) {
		queue_curr_sum += cnt;
	}
	else if (queue_curr_len < len) {
		queue_curr_len = len;
		queue_curr_sum = cnt;
	}
}

void queue_pop() {
	queue_sub++;
	if (queue[queue_head].skipped > 0) {
		queue[queue_head].skipped--;
	}
	else {
		queue_curr_sum -= queue[queue_head].cnt;
		queue_head++;
		if (queue_head < queue_tail && queue[queue_head].len != queue_curr_len) {
			queue_curr_len = queue[queue_head].len;
			for (int i = queue_head; i < queue_tail; i++) {
				if (queue[i].len == queue_curr_len)
					queue_curr_sum += queue[i].cnt;
				else break;
			}
		}
	}
}

void queue_register(int len, long long cnt) {
	register_paths(len + queue_curr_len - queue_sub, cnt * queue_curr_sum);
}

int main() {
	int n;
	int m;

	scanf("%d %d", &n, &m);
	for (int i = 0; i < m; i++) {
		int a;
		int b;
		scanf("%d %d", &a, &b);
		graph[a].push_back(b);
		graph[b].push_back(a);
	}

	if (n == m) {
		fetch_cycle();
		for (int i = 0; i < cycle_len; i++) {
			del_parent_edges(cycle[i]);
			get_paths(cycle[i]);
		}
	}
	else {
		del_parent_edges(1);
		get_paths(1);
	}

	for (int i = 1; i <= n; i++)
		register_subtree(i);

	for (int i = 1; i <= cycle_len / 2; i++)
		queue_push(path_len[cycle[i]], path_cnt[cycle[i]]);

	for (int i = 0; i < cycle_len; i++) {
		int ii = (i + (cycle_len / 2) + 1) % cycle_len;
		queue_register(path_len[cycle[i]], path_cnt[cycle[i]]);
		queue_push(path_len[cycle[ii]], path_cnt[cycle[ii]]);
		queue_pop();
	}

	printf("%d\n%lld\n", ans_len, ans_cnt);
	return 0;
}
